/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal.actions;

import java.util.Collection;
import java.util.LinkedList;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.fortiss.tooling.kernel.ui.extension.base.EObjectActionBase;
import org.fortiss.tooling.kernel.ui.util.CopyPasteUtils;

/**
 * Action for copying the current selection.
 * 
 * @author hummel
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash: 9F38A7C2C45764E90BFEEA02CC7B556A
 */
public class CopyAction extends EObjectActionBase {

	/** Constructor. */
	public CopyAction() {
		super("Copy", PlatformUI.getWorkbench().getSharedImages()
				.getImageDescriptor(ISharedImages.IMG_TOOL_COPY));
		setId(ActionFactory.COPY.getId());
	}

	/** {@inheritDoc} */
	@Override
	public void run() {
		if (getTarget() == null) {
			return;
		}
		// TODO @review CD: Can this work for several selected EObjects as well?
		// I do not think so.
		Collection<EObject> targets = new LinkedList<EObject>();
		targets.add(getTarget());
		CopyPasteUtils.copyToClipboard(targets);
	}

	/** {@inheritDoc} */
	@Override
	protected boolean computeEnabled() {
		return true;
	}
}
