/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Display;
import org.fortiss.tooling.kernel.extension.data.IElementCompositionContext;
import org.fortiss.tooling.kernel.service.IElementCompositorService;
import org.fortiss.tooling.kernel.utils.EMFResourceUtils;

/**
 * Methods for copy/paste transfer. EMF serialization and {@link TextTransfer}
 * is used to transfer an {@link EObject} to and from the clipboard.
 * 
 * @author hoelzl
 * @author hummel
 * 
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash: 7C1097EDDAA7883A5017CA83D37F5351
 */
public class CopyPasteUtils {

	/** Copies the given selected objects to the clipboard. */
	public static void copyToClipboard(Collection<EObject> selection) {
		try {
			ByteArrayOutputStream bytes = new ByteArrayOutputStream();
			// TODO @review CD: magic string
			Resource resource = new ResourceSetImpl().createResource(URI
					.createPlatformResourceURI("file://does/not/exist", true));
			for (EObject sel : selection) {
				resource.getContents().add(EcoreUtil.copy(sel));
			}
			resource.save(bytes, EMFResourceUtils.buildOptionsMap());
			String text = bytes.toString();

			Clipboard clipboard = new Clipboard(Display.getDefault());
			clipboard.setContents(new Object[] { text },
					new Transfer[] { TextTransfer.getInstance() },
					DND.CLIPBOARD | DND.SELECTION_CLIPBOARD);
			clipboard.dispose();
		} catch (Exception e) {
			MessageDialog.openError(Display.getDefault().getActiveShell(),
					"Copy",
					"Clipboard contents not writeable: " + e.getMessage());
			return;
		}
	}

	/**
	 * Attempts to paste the contents of the clipboard into the given target
	 * object. The given element composition context is forwarded to the element
	 * composition service.
	 */
	public static void pasteFromClipboard(EObject target,
			IElementCompositionContext context) {
		List<EObject> contents = getClipBoardContent();
		// TODO @review CD: unnecessary clone - either invoke canPasteInto or
		// omit and add in comment
		for (EObject insertObj : contents) {
			if (!IElementCompositorService.INSTANCE.canCompose(target,
					insertObj, context)) {
				return;
			}
		}
		for (EObject insertObj : contents) {
			IElementCompositorService.INSTANCE.compose(target, insertObj,
					context);
		}
	}

	/**
	 * Returns whether the current clipboard element is pasteable into the given
	 * target using the given composition context.
	 */
	public static boolean canPasteInto(EObject target,
			IElementCompositionContext context) {
		List<EObject> contents = getClipBoardContent();
		for (EObject insertObj : contents) {
			if (!IElementCompositorService.INSTANCE.canCompose(target,
					insertObj, context)) {
				return false;
			}
		}
		return true;
	}

	/** Returns elements read from clipboard. */
	private static List<EObject> getClipBoardContent() {
		Clipboard clipboard = new Clipboard(Display.getDefault());
		String text = (String) clipboard
				.getContents(TextTransfer.getInstance());
		clipboard.dispose();

		if (text == null) {
			return Collections.emptyList();
		}

		List<EObject> contents = new ArrayList<EObject>();
		try {
			ByteArrayInputStream in = new ByteArrayInputStream(text.getBytes());
			// TODO @review CD: magic string
			Resource resource = new ResourceSetImpl().createResource(URI
					.createPlatformResourceURI("file://does/not/exist", true));
			resource.load(in, EMFResourceUtils.buildOptionsMap());
			contents.addAll(resource.getContents());
		} catch (Exception e) {
			MessageDialog.openError(Display.getDefault().getActiveShell(),
					"Paste",
					"Clipboard contents not readable: " + e.getMessage());
			return Collections.emptyList();
		}
		return contents;
	}
}
